@@ -27,7 +27,7 @@ from astropy import wcs
 
 from . import wcs_utils
 from .spectral_cube import SpectralCube, VaryingResolutionSpectralCube, SIGMA2FWHM, np2wcs
-from .utils import cached, VarianceWarning, SliceWarning, BeamWarning, SmoothingWarning
+from .utils import cached, VarianceWarning, SliceWarning, BeamWarning, SmoothingWarning, BeamUnitsError
 from .lower_dimensional_structures import Projection
 from .masks import BooleanArrayMask, is_broadcastable_and_smaller
 from .np_compat import allbadtonan
@@ -36,6 +36,7 @@ __all__ = ['DaskSpectralCube', 'DaskVaryingResolutionSpectralCube']
 
 try:
     from scipy import ndimage
+    import scipy.interpolate
     SCIPY_INSTALLED = True
 except ImportError:
     SCIPY_INSTALLED = False
@@ -182,21 +183,24 @@ class FilledArrayHandler:
     """
 
     def __init__(self, cube, fill=np.nan):
-        self._cube = cube
+        self._data = cube._data
+        self._mask = cube._mask
         self._fill = fill
+        self._wcs = cube._wcs
+        self._wcs_tolerance = cube._wcs_tolerance
         self.shape = cube._data.shape
         self.dtype = cube._data.dtype
         self.ndim = len(self.shape)
 
     def __getitem__(self, view):
-        if self._cube._data[view].size == 0:
+        if self._data[view].size == 0:
             return 0.
         else:
-            return self._cube._mask._filled(data=self._cube._data,
-                                            view=view,
-                                            wcs=self._cube._wcs,
-                                            fill=self._fill,
-                                            wcs_tolerance=self._cube._wcs_tolerance)
+            return self._mask._filled(data=self._data,
+                                      view=view,
+                                      wcs=self._wcs,
+                                      fill=self._fill,
+                                      wcs_tolerance=self._wcs_tolerance)
 
 
 class MaskHandler:
@@ -207,14 +211,14 @@ class MaskHandler:
     """
 
     def __init__(self, cube):
-        self._cube = cube
+        self._data = cube._data
         self._mask = cube.mask
         self.shape = cube._data.shape
         self.dtype = cube._data.dtype
         self.ndim = len(self.shape)
 
     def __getitem__(self, view):
-        if self._cube._data[view].size == 0:
+        if self._data[view].size == 0:
             return False
         else:
             result = self._mask.include(view=view)
@@ -526,6 +530,7 @@ class DaskSpectralCubeMixin:
     def apply_function_parallel_spectral(self,
                                          function,
                                          accepts_chunks=False,
+                                         return_new_cube=True,
                                          **kwargs):
         """
         Apply a function in parallel along the spectral dimension.  The
@@ -550,19 +555,47 @@ class DaskSpectralCubeMixin:
             especially if carrying out several operations sequentially. If
             `False`, the computation is only carried out when accessing
             specific parts of the data or writing to disk.
+        return_new_cube : bool
+            If `True`, a new `~SpectralCube` object will be returned. This is the default
+            for when the function will return another version of the new spectral cube
+            with the operation applied (for example, spectral smoothing). If `False`,
+            an array will be returned from `function`. This is useful, for example,
+            when fitting a model to spectra and the output is the fitted model parameters.
         kwargs : dict
             Passed to ``function``
         """
 
-        def wrapper(data, **kwargs):
-            if data.size > 0:
-                return function(data, **kwargs)
-            else:
-                return data
+        # NOTE: `block_info` should always be available for `dask.array.map_blocks` to pass to
+        # Because we use this wrapper, this always should be an available kwarg, then we check
+        # if that kwarg should be passed to `function`
+        _has_blockinfo = 'block_info' in inspect.signature(function).parameters
+
+        # if/else to avoid an if/else in every single wrapper call.
+        if _has_blockinfo:
+            def wrapper(data, block_info=None, **kwargs):
+                if data.size > 0:
+                    return function(data, block_info=block_info, **kwargs)
+                else:
+                    return data
+        else:
+            def wrapper(data, **kwargs):
+                if data.size > 0:
+                    return function(data, **kwargs)
+                else:
+                    return data
 
         if accepts_chunks:
+            # Check if the spectral axis is already one chunk. If it is, there is no need to rechunk the data
+            current_chunksize = self._data.chunksize
+            if current_chunksize[0] == self.shape[0]:
+                rechunk = None
+            else:
+                rechunk = (-1, 'auto', 'auto')
+
             return self._map_blocks_to_cube(wrapper,
-                                            rechunk=(-1, 'auto', 'auto'), **kwargs)
+                                            return_new_cube=return_new_cube,
+                                            rechunk=rechunk, **kwargs)
+
         else:
             data = self._get_filled_data(fill=self._fill_value)
             # apply_along_axis returns an array with a single chunk, but we
@@ -570,11 +603,16 @@ class DaskSpectralCubeMixin:
             # even if it results in a poorer performance.
             data = data.rechunk((-1, 'auto', 'auto'))
             newdata = da.apply_along_axis(wrapper, 0, data, shape=(self.shape[0],))
-            return self._new_cube_with(data=newdata,
-                                       wcs=self.wcs,
-                                       mask=self.mask,
-                                       meta=self.meta,
-                                       fill_value=self.fill_value)
+
+            if return_new_cube:
+                return self._new_cube_with(data=newdata,
+                                        wcs=self.wcs,
+                                        mask=self.mask,
+                                        meta=self.meta,
+                                        fill_value=self.fill_value)
+            else:
+                return newdata
+
 
     @projection_if_needed
     @ignore_warnings
@@ -707,7 +745,57 @@ class DaskSpectralCubeMixin:
         """
         return self._compute(da.nanargmin(self._get_filled_data(fill=np.inf), axis=axis))
 
-    def _map_blocks_to_cube(self, function, additional_arrays=None, fill=np.nan, rechunk=None, **kwargs):
+    @ignore_warnings
+    def statistics(self):
+        """
+        Return a dictinary of global basic statistics for the data.
+
+        This method is designed to minimize the number of times each chunk is
+        accessed. The statistics are computed for each chunk in turn before
+        being aggregated.
+
+        The names for each statistic are adopted from CASA's ia.statistics
+        (see https://casa.nrao.edu/Release4.1.0/doc/CasaRef/image.statistics.html)
+        """
+
+        data = self._get_filled_data(fill=np.nan)
+
+        try:
+            from bottleneck import nanmin, nanmax, nansum
+        except ImportError:
+            from numpy import nanmin, nanmax, nansum
+
+        def compute_stats(chunk, *args):
+            return np.array([np.sum(~np.isnan(chunk)),
+                             nanmin(chunk),
+                             nanmax(chunk),
+                             nansum(chunk),
+                             nansum(chunk * chunk)])
+
+        with dask.config.set(**self._scheduler_kwargs):
+            results = da.map_blocks(compute_stats, data.reshape(-1)).compute()
+
+        count_values, min_values, max_values, sum_values, ssum_values = results.reshape((-1, 5)).T
+
+        stats = {'npts': count_values.sum(),
+                 'min': min_values.min() * self._unit,
+                 'max': max_values.max() * self._unit,
+                 'sum': sum_values.sum() * self._unit,
+                 'sumsq': ssum_values.sum() * self._unit ** 2}
+
+        stats['mean'] = stats['sum'] / stats['npts']
+
+        # FIXME: for now this uses the simple 'textbook' algorithm which is not
+        # numerically stable, so this should be replaced by a more robust approach
+        stats['sigma'] = ((stats['sumsq'] - stats['sum'] ** 2 / stats['npts']) / (stats['npts'] - 1)) ** 0.5
+
+        stats['rms'] = np.sqrt(stats['sumsq'] / stats['npts'])
+
+        return stats
+
+    def _map_blocks_to_cube(self, function, additional_arrays=None, fill=np.nan, rechunk=None,
+                            return_new_cube=True,
+                            **kwargs):
         """
         Call dask's map_blocks, returning a new spectral cube.
         """
@@ -718,19 +806,22 @@ class DaskSpectralCubeMixin:
             data = data.rechunk(rechunk)
 
         if additional_arrays is None:
-            newdata = data.map_blocks(function, dtype=data.dtype, **kwargs)
+            newdata = da.map_blocks(function, data, dtype=data.dtype, **kwargs)
         else:
             additional_arrays = [array.rechunk(data.chunksize) for array in additional_arrays]
             newdata = da.map_blocks(function, data, *additional_arrays, dtype=data.dtype, **kwargs)
 
         # Create final output cube
-        newcube = self._new_cube_with(data=newdata,
-                                      wcs=self.wcs,
-                                      mask=self.mask,
-                                      meta=self.meta,
-                                      fill_value=self.fill_value)
+        if return_new_cube:
+            newcube = self._new_cube_with(data=newdata,
+                                          wcs=self.wcs,
+                                          mask=self.mask,
+                                          meta=self.meta,
+                                          fill_value=self.fill_value)
 
-        return newcube
+            return newcube
+        else:
+            return newdata
 
     # NOTE: the following three methods could also be implemented spaxel by
     # spaxel using apply_function_parallel_spectral but then take longer (but
@@ -760,8 +851,8 @@ class DaskSpectralCubeMixin:
         """
 
         def spectral_sigma_clip(array):
-            result = stats.sigma_clip(array, sigma=threshold, axis=0, **kwargs)
-            return result.filled(np.nan)
+            return stats.sigma_clip(array, sigma=threshold, axis=0,
+                                    masked=False, copy=True, **kwargs)
 
         return self.apply_function_parallel_spectral(spectral_sigma_clip,
                                                      accepts_chunks=True)
@@ -806,7 +897,7 @@ class DaskSpectralCubeMixin:
                                                      accepts_chunks=True)
 
     @add_save_to_tmp_dir_option
-    def spectral_smooth_median(self, ksize, **kwargs):
+    def spectral_smooth_median(self, ksize, raise_error_jybm=True, **kwargs):
         """
         Smooth the cube along the spectral dimension
 
@@ -839,7 +930,7 @@ class DaskSpectralCubeMixin:
                                                      accepts_chunks=True)
 
     @add_save_to_tmp_dir_option
-    def spatial_smooth(self, kernel, convolve=convolution.convolve, **kwargs):
+    def spatial_smooth(self, kernel, convolve=convolution.convolve, raise_error_jybm=True, **kwargs):
         """
         Smooth the image in each spatial-spatial plane of the cube.
 
@@ -851,6 +942,9 @@ class DaskSpectralCubeMixin:
             The astropy convolution function to use, either
             `astropy.convolution.convolve` or
             `astropy.convolution.convolve_fft`
+        raise_error_jybm : bool, optional
+            Raises a `~spectral_cube.utils.BeamUnitsError` when smoothing a cube in Jy/beam units,
+            since the brightness is dependent on the spatial resolution.
         save_to_tmp_dir : bool
             If `True`, the computation will be carried out straight away and
             saved to a temporary directory. This can improve performance,
@@ -861,13 +955,15 @@ class DaskSpectralCubeMixin:
             Passed to the convolve function
         """
 
+        self.check_jybeam_smoothing(raise_error_jybm=raise_error_jybm)
+
         def convolve_wrapper(data, kernel=None, **kwargs):
             return convolve(data, kernel, normalize_kernel=True, **kwargs)
 
         return self.apply_function_parallel_spatial(convolve_wrapper, kernel=kernel.array)
 
     @add_save_to_tmp_dir_option
-    def spatial_smooth_median(self, ksize, **kwargs):
+    def spatial_smooth_median(self, ksize, raise_error_jybm=True, **kwargs):
         """
         Smooth the image in each spatial-spatial plane of the cube using a median filter.
 
@@ -875,6 +971,9 @@ class DaskSpectralCubeMixin:
         ----------
         ksize : int
             Size of the median filter (scipy.ndimage.filters.median_filter)
+        raise_error_jybm : bool, optional
+            Raises a `~spectral_cube.utils.BeamUnitsError` when smoothing a cube in Jy/beam units,
+            since the brightness is dependent on the spatial resolution.
         kwargs : dict
             Passed to the median_filter function
         """
@@ -882,8 +981,10 @@ class DaskSpectralCubeMixin:
         if not SCIPY_INSTALLED:
             raise ImportError("Scipy could not be imported: this function won't work.")
 
-        def median_filter_wrapper(data, ksize=None):
-            return ndimage.median_filter(data, ksize)
+        self.check_jybeam_smoothing(raise_error_jybm=raise_error_jybm)
+
+        def median_filter_wrapper(data, ksize=None, **kwargs):
+            return ndimage.median_filter(data, ksize, **kwargs)
 
         return self.apply_function_parallel_spatial(median_filter_wrapper, ksize=ksize)
 
@@ -1175,21 +1276,24 @@ class DaskSpectralCubeMixin:
             warnings.warn("Input grid has too small a spacing. The data should "
                           "be smoothed prior to resampling.", SmoothingWarning)
 
+        if reverse_in:
+            cubedata = cubedata[::-1, :, :]
+
+        cubedata = cubedata.rechunk((-1, 'auto', 'auto'))
+        chunkshape = (len(spectral_grid),) + cubedata.chunksize[1:]
+
         def interp_wrapper(y, args):
             if y.size == 1:
                 return y
             else:
-                return np.interp(args[0], args[1], y[:, 0, 0],
-                                 left=fill_value, right=fill_value).reshape((-1, 1, 1))
-
-        if reverse_in:
-            cubedata = cubedata[::-1, :, :]
-
-        cubedata = cubedata.rechunk((-1, 1, 1))
+                interp = scipy.interpolate.interp1d(args[1], y.T,
+                                                    fill_value=fill_value,
+                                                    bounds_error=False)
+                return interp(args[0]).T
 
         newcube = cubedata.map_blocks(interp_wrapper,
                                       args=(spectral_grid.value, inaxis.value),
-                                      chunks=(len(spectral_grid), 1, 1))
+                                      chunks=chunkshape)
 
         newwcs = self.wcs.deepcopy()
         newwcs.wcs.crpix[2] = 1
@@ -1286,13 +1390,18 @@ class DaskSpectralCube(DaskSpectralCubeMixin, SpectralCube):
         convolution_kernel = beam.deconvolve(self.beam).as_kernel(pixscale)
         kernel = convolution_kernel.array.reshape((1,) + convolution_kernel.array.shape)
 
+        if self.unit.is_equivalent(u.Jy / u.beam):
+            beam_ratio_factor = (beam.sr / self.beam.sr).value
+        else:
+            beam_ratio_factor = 1.
+
         # See #631: kwargs get passed within self.apply_function_parallel_spatial
         def convfunc(img, **kwargs):
-            return convolve(img, kernel, normalize_kernel=True, **kwargs).reshape(img.shape)
+            return convolve(img, kernel, normalize_kernel=True, **kwargs).reshape(img.shape) * beam_ratio_factor
 
         return self.apply_function_parallel_spatial(convfunc,
                                                     accepts_chunks=True,
-                                                    **kwargs).with_beam(beam)
+                                                    **kwargs).with_beam(beam, raise_error_jybm=False)
 
 
 class DaskVaryingResolutionSpectralCube(DaskSpectralCubeMixin, VaryingResolutionSpectralCube):
@@ -1395,20 +1504,25 @@ class DaskVaryingResolutionSpectralCube(DaskSpectralCubeMixin, VaryingResolution
         pixscale = wcs.utils.proj_plane_pixel_area(self.wcs.celestial)**0.5*u.deg
 
         beams = []
+        beam_ratio_factors = []
         for bm, valid in zip(self.unmasked_beams, self.goodbeams_mask):
             if not valid:
                 # just skip masked-out beams
                 beams.append(None)
+                beam_ratio_factors.append(None)
                 continue
             elif beam == bm:
                 # Point response when beams are equal, don't convolve.
                 beams.append(None)
+                beam_ratio_factors.append(None)
                 continue
             try:
                 beams.append(beam.deconvolve(bm))
+                beam_ratio_factors.append((beam.sr / bm.sr).value)
             except ValueError:
                 if allow_smaller:
                     beams.append(None)
+                    beam_ratio_factors.append(None)
                 else:
                     raise
 
@@ -1417,6 +1531,8 @@ class DaskVaryingResolutionSpectralCube(DaskSpectralCubeMixin, VaryingResolution
         beams = da.from_array(np.array(beams, dtype=np.object)
                               .reshape((len(beams), 1, 1)), chunks=(-1, -1, -1))
 
+        needs_beam_ratio = self.unit.is_equivalent(u.Jy / u.beam)
+
         # See #631: kwargs get passed within self.apply_function_parallel_spatial
         def convfunc(img, beam, **kwargs):
             if img.size > 0:
@@ -1427,6 +1543,10 @@ class DaskVaryingResolutionSpectralCube(DaskSpectralCubeMixin, VaryingResolution
                     else:
                         kernel = beam[index, 0, 0].as_kernel(pixscale)
                         out[index] = convolve(img[index], kernel, normalize_kernel=True, **kwargs)
+
+                        if needs_beam_ratio:
+                            out[index] *= beam_ratio_factors[index]
+
                 return out
             else:
                 return img
@@ -1463,4 +1583,8 @@ class DaskVaryingResolutionSpectralCube(DaskSpectralCubeMixin, VaryingResolution
 
     @property
     def _mask_include(self):
-        return da.from_array(MaskHandler(self), name='MaskHandler ' + str(uuid.uuid4()), chunks=self._data.chunksize)
+        return BooleanArrayMask(da.from_array(MaskHandler(self),
+                                              name='MaskHandler ' + str(uuid.uuid4()),
+                                              chunks=self._data.chunksize),
+                                wcs=self.wcs,
+                                shape=self.shape)
