@@ -16,8 +16,8 @@ from astropy.io.registry import UnifiedReadWriteMethod
 
 from . import spectral_axis
 from .io.core import LowerDimensionalObjectWrite
-from .utils import SliceWarning, BeamWarning, SmoothingWarning, FITSWarning
-from .cube_utils import convert_bunit
+from .utils import SliceWarning, BeamWarning, SmoothingWarning, FITSWarning, BeamUnitsError
+from . import cube_utils
 from . import wcs_utils
 from .masks import BooleanArrayMask, MaskBase
 
@@ -26,7 +26,6 @@ from .base_class import (BaseNDClass, SpectralAxisMixinClass,
                          MultiBeamMixinClass, BeamMixinClass,
                          HeaderMixinClass
                         )
-from . import cube_utils
 
 __all__ = ['LowerDimensionalObject', 'Projection', 'Slice', 'OneDSpectrum']
 class LowerDimensionalObject(u.Quantity, BaseNDClass, HeaderMixinClass):
@@ -162,48 +161,15 @@ class LowerDimensionalObject(u.Quantity, BaseNDClass, HeaderMixinClass):
             # No copying
             return self
 
-        if ((self.unit.is_equivalent(u.Jy / u.beam) and
-             not any({u.Jy/u.beam, u.K}.issubset(set(eq)) for eq in equivalencies))):
-            # the 'not any' above checks that there is not already a defined
-            # Jy<->K equivalency.  If there is, the code below is redundant
-            # and will cause problems.
+        if hasattr(self, 'with_spectral_unit'):
+            freq = self.with_spectral_unit(u.Hz).spectral_axis
 
-            if hasattr(self, 'beams'):
-                factor = (self.jtok_factors(equivalencies=equivalencies) *
-                          (self.unit*u.beam).to(u.Jy))
-            else:
-                # replace "beam" with the actual beam
-                if not hasattr(self, 'beam'):
-                    raise ValueError("To convert objects with Jy/beam units, "
-                                     "the object needs to have a beam defined.")
-                brightness_unit = self.unit * u.beam
+        if freq is None and 'RESTFRQ' in self.header:
+            freq = self.header['RESTFRQ'] * u.Hz
 
-                # create a beam equivalency for brightness temperature
-                if freq is None:
-                    try:
-                        freq = self.with_spectral_unit(u.Hz).spectral_axis
-                    except AttributeError:
-                        raise TypeError("Object of type {0} has no spectral "
-                                        "information. `freq` must be provided for"
-                                        " unit conversion from Jy/beam"
-                                        .format(type(self)))
-                else:
-                    if not freq.unit.is_equivalent(u.Hz):
-                        raise u.UnitsError("freq must be given in equivalent "
-                                           "frequency units.")
-
-                bmequiv = self.beam.jtok_equiv(freq)
-                # backport to handle astropy < 3: the beam equivalency was only
-                # modified to handle jy/beam in astropy 3
-                if bmequiv[0] == u.Jy:
-                    bmequiv.append([u.Jy/u.beam, u.K, bmequiv[2], bmequiv[3]])
-
-                factor = brightness_unit.to(unit,
-                                            equivalencies=bmequiv + list(equivalencies))
-
-        else:
-            # scaling factor
-            factor = self.unit.to(unit, equivalencies=equivalencies)
+        # Create the tuple of unit conversions needed.
+        factor = cube_utils.bunit_converters(self, unit, equivalencies=equivalencies,
+                                             freq=freq)
 
         converted_array = (self.quantity * factor).value
 
@@ -317,7 +283,7 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
 
         return self
 
-    def with_beam(self, beam):
+    def with_beam(self, beam, raise_error_jybm=True):
         '''
         Attach a new beam object to the Projection.
 
@@ -327,6 +293,11 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
             A new beam object.
         '''
 
+        if not isinstance(beam, Beam):
+            raise TypeError("beam must be a radio_beam.Beam object.")
+
+        self.check_jybeam_smoothing(raise_error_jybm=raise_error_jybm)
+
         meta = self.meta.copy()
         meta['beam'] = beam
 
@@ -412,7 +383,7 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
         mywcs = wcs.WCS(hdu.header)
 
         if "BUNIT" in hdu.header:
-            unit = convert_bunit(hdu.header["BUNIT"])
+            unit = cube_utils.convert_bunit(hdu.header["BUNIT"])
             meta["BUNIT"] = hdu.header["BUNIT"]
         else:
             unit = None
@@ -567,6 +538,11 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
         """
         Extract a region spatially.
 
+        When spatial WCS dimensions are given as an `~astropy.units.Quantity`,
+        the spatial coordinates of the 'lo' and 'hi' corners are solved together.
+        This minimizes WCS variations due to the sky curvature when slicing from
+        a large (>1 deg) image.
+
         Parameters
         ----------
         [xy]lo/[xy]hi : int or `astropy.units.Quantity` or `min`/`max`
@@ -577,36 +553,8 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
 
         self._raise_wcs_no_celestial()
 
-        limit_dict = {'xlo': 0 if xlo == 'min' else xlo,
-                      'ylo': 0 if ylo == 'min' else ylo,
-                      'xhi': self.shape[1] if xhi == 'max' else xhi,
-                      'yhi': self.shape[0] if yhi == 'max' else yhi}
-        dims = {'x': 1,
-                'y': 0}
-
-        for val in (xlo, ylo, xhi, yhi):
-            if hasattr(val, 'unit') and not val.unit.is_equivalent(u.degree):
-                raise u.UnitsError("The X and Y slices must be specified in "
-                                   "degree-equivalent units.")
-
-        for lim in limit_dict:
-            limval = limit_dict[lim]
-            if hasattr(limval, 'unit'):
-                dim = dims[lim[0]]
-                sl = [slice(0, 1)]
-                sl.insert(dim, slice(None))
-                spine = self.world[tuple(sl)][dim]
-                val = np.argmin(np.abs(limval - spine))
-                if limval > spine.max() or limval < spine.min():
-                    pass
-                    # log.warn("The limit {0} is out of bounds."
-                    #          "  Using min/max instead.".format(lim))
-                if lim[1:] == 'hi':
-                    # End-inclusive indexing: need to add one for the high
-                    # slice
-                    limit_dict[lim] = val + 1
-                else:
-                    limit_dict[lim] = val
+        # Solve for the spatial pixel indices together
+        limit_dict = wcs_utils.find_spatial_pixel_index(self, xlo, xhi, ylo, yhi)
 
         slices = [slice(limit_dict[xx + 'lo'], limit_dict[xx + 'hi'])
                   for xx in 'yx']
@@ -696,7 +644,7 @@ class BaseOneDSpectrum(LowerDimensionalObject, MaskableArrayMixinClass,
         mywcs = wcs.WCS(hdu.header)
 
         if "BUNIT" in hdu.header:
-            unit = convert_bunit(hdu.header["BUNIT"])
+            unit = cube_utils.convert_bunit(hdu.header["BUNIT"])
             meta["BUNIT"] = hdu.header["BUNIT"]
         else:
             unit = None
@@ -1066,7 +1014,7 @@ class OneDSpectrum(BaseOneDSpectrum, BeamMixinClass):
         out = super(OneDSpectrum, self)._new_spectrum_with(beam=beam, **kwargs)
         return out
 
-    def with_beam(self, beam):
+    def with_beam(self, beam, raise_error_jybm=True):
         '''
         Attach a new beam object to the OneDSpectrum.
 
@@ -1076,6 +1024,11 @@ class OneDSpectrum(BaseOneDSpectrum, BeamMixinClass):
             A new beam object.
         '''
 
+        if not isinstance(beam, Beam):
+            raise TypeError("beam must be a radio_beam.Beam object.")
+
+        self.check_jybeam_smoothing(raise_error_jybm=raise_error_jybm)
+
         meta = self.meta.copy()
         meta['beam'] = beam
 
