@@ -225,7 +225,7 @@ class MaskBase(object):
         """
         # Must convert to floating point, but should not change from inherited
         # type otherwise
-        dt = np.find_common_type([data.dtype], [np.float])
+        dt = np.find_common_type([data.dtype], [float])
 
         if use_memmap and data.size > 0:
             ntf = tempfile.NamedTemporaryFile()
@@ -256,6 +256,18 @@ class MaskBase(object):
         raise NotImplementedError("{0} mask classes do not have shape attributes."
                                   .format(self.__class__.__name__))
 
+    @property
+    def ndim(self):
+        return len(self.shape)
+
+    @property
+    def size(self):
+        return np.product(self.shape)
+
+    @property
+    def dtype(self):
+        return np.dtype('bool')
+
     def __getitem__(self):
         raise NotImplementedError("Slicing not supported by mask class {0}"
                                   .format(self.__class__.__name__))
