@@ -1,5 +1,7 @@
 """
-Performance-related tests to make sure we don't use more memory than we should
+Performance-related tests to make sure we don't use more memory than we should.
+
+For now this is just for SpectralCube, not DaskSpectralCube.
 """
 
 from __future__ import print_function, absolute_import, division
@@ -29,12 +31,16 @@ from . import utilities
 
 from astropy import convolution, units as u
 
+WINDOWS = sys.platform == "win32"
+
+
 def find_base_nbytes(obj):
     # from http://stackoverflow.com/questions/34637875/size-of-numpy-strided-array-broadcast-array-in-memory
     if obj.base is not None:
         return find_base_nbytes(obj.base)
     return obj.nbytes
 
+
 def test_pix_size():
     mc_hdu = moment_cube()
     sc = SpectralCube.read(mc_hdu)
@@ -48,6 +54,7 @@ def test_pix_size():
     assert find_base_nbytes(y) == sc.shape[1]*sc.shape[2]*bytes_per_pix
     assert find_base_nbytes(x) == sc.shape[1]*sc.shape[2]*bytes_per_pix
 
+
 def test_compare_pix_size_approaches():
     mc_hdu = moment_cube()
     sc = SpectralCube.read(mc_hdu)
@@ -59,6 +66,7 @@ def test_compare_pix_size_approaches():
     assert_allclose(ya, y)
     assert_allclose(xa, x)
 
+
 def test_pix_cen():
     mc_hdu = moment_cube()
     sc = SpectralCube.read(mc_hdu)
@@ -72,6 +80,7 @@ def test_pix_cen():
     assert find_base_nbytes(y) == sc.shape[1]*sc.shape[2]*bytes_per_pix
     assert find_base_nbytes(x) == sc.shape[1]*sc.shape[2]*bytes_per_pix
 
+
 @pytest.mark.skipif('True')
 def test_parallel_performance_smoothing():
 
@@ -125,7 +134,7 @@ def test_parallel_performance_smoothing():
             print(rslt)
 
 # python 2.7 doesn't have tracemalloc
-@pytest.mark.skipif('not tracemallocOK or (sys.version_info.major==3 and sys.version_info.minor<6) or not NPY_VERSION_CHECK')
+@pytest.mark.skipif('not tracemallocOK or (sys.version_info.major==3 and sys.version_info.minor<6) or not NPY_VERSION_CHECK or WINDOWS')
 def test_memory_usage():
     """
     Make sure that using memmaps happens where expected, for the most part, and
@@ -157,6 +166,7 @@ def test_memory_usage():
 
     # writing the cube should not occupy any more memory
     snap3 = tracemalloc.take_snapshot()
+
     diff = snap3.compare_to(snap2, 'lineno')
     assert sum([dd.size_diff for dd in diff])*u.B < 100*u.kB
 
