@@ -16,7 +16,7 @@ from ..spectral_cube import SpectralCube
 from ..masks import BooleanArrayMask
 from ..lower_dimensional_structures import (Projection, Slice, OneDSpectrum,
                                             VaryingResolutionOneDSpectrum)
-from ..utils import SliceWarning, WCSCelestialError
+from ..utils import SliceWarning, WCSCelestialError, BeamUnitsError
 from . import path
 
 # set up for parametrization
@@ -427,6 +427,26 @@ def test_ldo_attach_beam(LDO, data):
     assert new_p.meta['beam'] == newbeam
 
 
+
+@pytest.mark.xfail(raises=BeamUnitsError, strict=True)
+@pytest.mark.parametrize(('LDO', 'data'),
+                         zip(LDOs, data_twelve))
+def test_ldo_attach_beam_jybm_error(LDO, data):
+
+    exp_beam = Beam(1.0 * u.arcsec)
+    newbeam = Beam(2.0 * u.arcsec)
+
+    data = data.value * u.Jy / u.beam
+
+    p = LDO(data, copy=False, beam=exp_beam)
+
+    # Attaching with no beam should work.
+    new_p = p.with_beam(newbeam)
+
+    # Trying to change the beam should now raise a BeamUnitsError
+    new_p = new_p.with_beam(newbeam)
+
+
 @pytest.mark.parametrize(('LDO', 'data'),
                          zip(LDOs_2d, data_two_2d))
 def test_projection_from_hdu(LDO, data):
@@ -447,20 +467,48 @@ def test_projection_subimage(data_55):
     proj1 = proj.subimage(xlo=1, xhi=3)
     proj2 = proj.subimage(xlo=24.06269 * u.deg,
                           xhi=24.06206 * u.deg)
+    proj3 = proj.subimage(xlo=24.06269*u.deg, xhi=3)
+    proj4 = proj.subimage(xlo=1, xhi=24.06206*u.deg)
 
     assert proj1.shape == (5, 2)
     assert proj2.shape == (5, 2)
+    assert proj3.shape == (5, 2)
+    assert proj4.shape == (5, 2)
     assert proj1.wcs.wcs.compare(proj2.wcs.wcs)
+    assert proj1.wcs.wcs.compare(proj3.wcs.wcs)
+    assert proj1.wcs.wcs.compare(proj4.wcs.wcs)
     assert proj.beam == proj1.beam
     assert proj.beam == proj2.beam
 
-    proj3 = proj.subimage(ylo=1, yhi=3)
-    proj4 = proj.subimage(ylo=29.93464 * u.deg,
+    proj4 = proj.subimage(ylo=1, yhi=3)
+    proj5 = proj.subimage(ylo=29.93464 * u.deg,
                           yhi=29.93522 * u.deg)
+    proj6 = proj.subimage(ylo=1, yhi=29.93522 * u.deg)
+    proj7 = proj.subimage(ylo=29.93464 * u.deg, yhi=3)
 
-    assert proj3.shape == (2, 5)
     assert proj4.shape == (2, 5)
-    assert proj3.wcs.wcs.compare(proj4.wcs.wcs)
+    assert proj5.shape == (2, 5)
+    assert proj6.shape == (2, 5)
+    assert proj7.shape == (2, 5)
+    assert proj4.wcs.wcs.compare(proj5.wcs.wcs)
+    assert proj4.wcs.wcs.compare(proj6.wcs.wcs)
+    assert proj4.wcs.wcs.compare(proj7.wcs.wcs)
+
+    # Test mixed slicing in both spatial directions
+    proj1xy = proj.subimage(xlo=1, xhi=3, ylo=1, yhi=3)
+    proj2xy = proj.subimage(xlo=24.06269*u.deg, xhi=3,
+                            ylo=1,yhi=29.93522 * u.deg)
+
+    proj3xy = proj.subimage(xlo=1, xhi=24.06206*u.deg,
+                            ylo=29.93464 * u.deg, yhi=3)
+
+
+    assert proj1xy.shape == (2, 2)
+    assert proj2xy.shape == (2, 2)
+    assert proj3xy.shape == (2, 2)
+    assert proj1xy.wcs.wcs.compare(proj2xy.wcs.wcs)
+    assert proj1xy.wcs.wcs.compare(proj3xy.wcs.wcs)
+
 
     proj5 = proj.subimage()
 
@@ -716,6 +764,81 @@ def test_beam_jtok_2D(data_advs, use_dask):
     np.testing.assert_almost_equal(Kplane.value,
                                    (plane.value * jtok).value)
 
+bunits_list = [u.Jy / u.beam, u.K, u.Jy / u.sr, u.Jy / u.pix, u.Jy / u.arcsec**2,
+               u.mJy / u.beam, u.mK]
+
+@pytest.mark.parametrize(('init_unit'), bunits_list)
+def test_unit_conversions_general_2D(data_advs, use_dask, init_unit):
+
+    cube, data = cube_and_raw(data_advs, use_dask=use_dask)
+    cube._meta['BUNIT'] = init_unit.to_string()
+    cube._unit = init_unit
+
+    plane = cube[0]
+
+    # Check all unit conversion combos:
+    for targ_unit in bunits_list:
+        newplane = plane.to(targ_unit)
+
+        if init_unit == targ_unit:
+            np.testing.assert_almost_equal(newplane.value,
+                                           plane.value)
+
+        else:
+            roundtrip_plane = newplane.to(init_unit)
+            np.testing.assert_almost_equal(roundtrip_plane.value,
+                                           plane.value)
+
+# TODO: Our 1D object do NOT retain spatial info that is needed for other BUNIT conversion
+# e.g., Jy/sr, Jy/pix. So we're limited to Jy/beam -> K conversion for now
+# See: https://github.com/radio-astro-tools/spectral-cube/pull/395
+bunits_list_1D = [u.Jy / u.beam, u.K,
+                  u.mJy / u.beam, u.mK]
+
+@pytest.mark.parametrize(('init_unit'), bunits_list_1D)
+def test_unit_conversions_general_1D(data_advs, use_dask, init_unit):
+
+    cube, data = cube_and_raw(data_advs, use_dask=use_dask)
+    cube._meta['BUNIT'] = init_unit.to_string()
+    cube._unit = init_unit
+
+    spec = cube[:, 0, 0]
+
+    # Check all unit conversion combos:
+    for targ_unit in bunits_list_1D:
+        newspec = spec.to(targ_unit)
+
+        if init_unit == targ_unit:
+            np.testing.assert_almost_equal(newspec.value,
+                                           spec.value)
+
+        else:
+            roundtrip_spec = newspec.to(init_unit)
+            np.testing.assert_almost_equal(roundtrip_spec.value,
+                                           spec.value)
+
+@pytest.mark.parametrize(('init_unit'), bunits_list_1D)
+def test_multibeams_unit_conversions_general_1D(data_vda_beams, use_dask, init_unit):
+
+    cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
+    cube._meta['BUNIT'] = init_unit.to_string()
+    cube._unit = init_unit
+
+    spec = cube[:, 0, 0]
+
+    # Check all unit conversion combos:
+    for targ_unit in bunits_list_1D:
+        newspec = spec.to(targ_unit)
+
+        if init_unit == targ_unit:
+            np.testing.assert_almost_equal(newspec.value,
+                                           spec.value)
+
+        else:
+            roundtrip_spec = newspec.to(init_unit)
+            np.testing.assert_almost_equal(roundtrip_spec.value,
+                                           spec.value)
+
 
 def test_basic_arrayness(data_adv, use_dask):
     cube, data = cube_and_raw(data_adv, use_dask=use_dask)
