@@ -29,9 +29,11 @@ from .test_spectral_cube import cube_and_raw
 from .test_projection import load_projection
 from . import path, utilities
 
+WINDOWS = sys.platform == "win32"
 
-def test_convolution():
-    cube, data = cube_and_raw('255_delta.fits')
+
+def test_convolution(data_255_delta, use_dask):
+    cube, data = cube_and_raw(data_255_delta, use_dask=use_dask)
 
     # 1" convolved with 1.5" -> 1.8027....
     target_beam = Beam(1.802775637731995*u.arcsec, 1.802775637731995*u.arcsec,
@@ -54,8 +56,8 @@ def test_convolution():
     assert np.all(conv_cube.filled_data[1,:,:] == 0.0)
 
 
-def test_beams_convolution():
-    cube, data = cube_and_raw('455_delta_beams.fits')
+def test_beams_convolution(data_455_delta_beams, use_dask):
+    cube, data = cube_and_raw(data_455_delta_beams, use_dask=use_dask)
 
     # 1" convolved with 1.5" -> 1.8027....
     target_beam = Beam(1.802775637731995*u.arcsec, 1.802775637731995*u.arcsec,
@@ -74,8 +76,8 @@ def test_beams_convolution():
                                        conv_cube.filled_data[ii,:,:].value)
 
 
-def test_beams_convolution_equal():
-    cube, data = cube_and_raw('522_delta_beams.fits')
+def test_beams_convolution_equal(data_522_delta_beams, use_dask):
+    cube, data = cube_and_raw(data_522_delta_beams, use_dask=use_dask)
 
     # Only checking that the equal beam case is handled correctly.
     # Fake the beam in the first channel. Then ensure that the first channel
@@ -92,11 +94,11 @@ def test_beams_convolution_equal():
 
 
 @pytest.mark.parametrize('use_memmap', (True, False))
-def test_reproject(use_memmap):
+def test_reproject(use_memmap, data_adv, use_dask):
 
     pytest.importorskip('reproject')
 
-    cube, data = cube_and_raw('adv.fits')
+    cube, data = cube_and_raw(data_adv, use_dask=use_dask)
 
     wcs_in = WCS(cube.header)
     wcs_out = wcs_in.deepcopy()
@@ -120,9 +122,9 @@ def test_reproject(use_memmap):
     assert result_wcs_from_header.wcs.compare(wcs_out.wcs)
 
 
-def test_spectral_smooth():
+def test_spectral_smooth(data_522_delta, use_dask):
 
-    cube, data = cube_and_raw('522_delta.fits')
+    cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
 
     result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), use_memmap=False)
 
@@ -138,22 +140,23 @@ def test_spectral_smooth():
                                                                 x_size=5).array,
                                    4)
 
-def test_catch_kernel_with_units():
+def test_catch_kernel_with_units(data_522_delta, use_dask):
     # Passing a kernel with a unit should raise a u.UnitsError
 
-    cube, data = cube_and_raw('522_delta.fits')
+    cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
 
-    with pytest.raises(u.UnitsError) as exc:
-        result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0 * u.dimensionless_unscaled),
-                                      use_memmap=False)
-    assert exc.value.args[0] == "The convolution kernel should be defined without a unit."
+    with pytest.raises(u.UnitsError,
+                       match="The convolution kernel should be defined without a unit."):
+        cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0 * u.one),
+                             use_memmap=False)
 
 
-def test_spectral_smooth_4cores():
+@pytest.mark.skipif('WINDOWS')
+def test_spectral_smooth_4cores(data_522_delta):
 
     pytest.importorskip('joblib')
 
-    cube, data = cube_and_raw('522_delta.fits')
+    cube, data = cube_and_raw(data_522_delta, use_dask=False)
 
     result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), num_cores=4, use_memmap=True)
 
@@ -172,13 +175,13 @@ def test_spectral_smooth_4cores():
 
     # num_cores = 4 is a contradiction with parallel=False, so we want to make
     # sure it fails
-    with pytest.raises(ValueError) as exc:
-        result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0),
-                                      num_cores=4, parallel=False)
-    assert exc.value.args[0] == ("parallel execution was not requested, but "
+    with pytest.raises(ValueError,
+                       match=("parallel execution was not requested, but "
                                  "multiple cores were: these are incompatible "
                                  "options.  Either specify num_cores=1 or "
-                                 "parallel=True")
+                                 "parallel=True")):
+        result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0),
+                                      num_cores=4, parallel=False)
 
     np.testing.assert_almost_equal(result[:,0,0].value,
                                    convolution.Gaussian1DKernel(1.0,
@@ -186,22 +189,21 @@ def test_spectral_smooth_4cores():
                                    4)
 
 
-def test_spectral_smooth_fail():
+def test_spectral_smooth_fail(data_522_delta_beams, use_dask):
 
-    cube, data = cube_and_raw('522_delta_beams.fits')
+    cube, data = cube_and_raw(data_522_delta_beams, use_dask=use_dask)
 
-    with pytest.raises(AttributeError) as exc:
+    with pytest.raises(AttributeError,
+                       match=("VaryingResolutionSpectralCubes can't be "
+                              "spectrally smoothed.  Convolve to a "
+                              "common resolution with `convolve_to` before "
+                              "attempting spectral smoothed.")):
         cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0))
 
-    assert exc.value.args[0] == ("VaryingResolutionSpectralCubes can't be "
-                                 "spectrally smoothed.  Convolve to a "
-                                 "common resolution with `convolve_to` before "
-                                 "attempting spectral smoothed.")
 
+def test_spectral_interpolate(data_522_delta, use_dask):
 
-def test_spectral_interpolate():
-
-    cube, data = cube_and_raw('522_delta.fits')
+    cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
 
     orig_wcs = cube.wcs.deepcopy()
 
@@ -216,9 +218,9 @@ def test_spectral_interpolate():
     assert cube.wcs.wcs.compare(orig_wcs.wcs)
 
 
-def test_spectral_interpolate_with_fillvalue():
+def test_spectral_interpolate_with_fillvalue(data_522_delta, use_dask):
 
-    cube, data = cube_and_raw('522_delta.fits')
+    cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
 
     # Step one channel out of bounds.
     sg = ((cube.spectral_axis[0]) -
@@ -230,27 +232,27 @@ def test_spectral_interpolate_with_fillvalue():
                                    np.ones(4)*42)
 
 
-def test_spectral_interpolate_fail():
+def test_spectral_interpolate_fail(data_522_delta_beams, use_dask):
 
-    cube, data = cube_and_raw('522_delta_beams.fits')
+    cube, data = cube_and_raw(data_522_delta_beams, use_dask=use_dask)
 
-    with pytest.raises(AttributeError) as exc:
+    with pytest.raises(AttributeError,
+                       match=("VaryingResolutionSpectralCubes can't be "
+                              "spectrally interpolated.  Convolve to a "
+                              "common resolution with `convolve_to` before "
+                              "attempting spectral interpolation.")):
         cube.spectral_interpolate(5)
 
-    assert exc.value.args[0] == ("VaryingResolutionSpectralCubes can't be "
-                                 "spectrally interpolated.  Convolve to a "
-                                 "common resolution with `convolve_to` before "
-                                 "attempting spectral interpolation.")
 
+def test_spectral_interpolate_with_mask(data_522_delta, use_dask):
 
-def test_spectral_interpolate_with_mask():
-
-    hdu = fits.open(path("522_delta.fits"))[0]
+    hdul = fits.open(data_522_delta)
+    hdu = hdul[0]
 
     # Swap the velocity axis so indiff < 0 in spectral_interpolate
     hdu.header["CDELT3"] = - hdu.header["CDELT3"]
 
-    cube = SpectralCube.read(hdu)
+    cube = SpectralCube.read(hdu, use_dask=use_dask)
 
     mask = np.ones(cube.shape, dtype=bool)
     mask[:2] = False
@@ -271,10 +273,12 @@ def test_spectral_interpolate_with_mask():
 
     assert cube.wcs.wcs.compare(orig_wcs.wcs)
 
+    hdul.close()
 
-def test_spectral_interpolate_reversed():
 
-    cube, data = cube_and_raw('522_delta.fits')
+def test_spectral_interpolate_reversed(data_522_delta, use_dask):
+
+    cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
 
     orig_wcs = cube.wcs.deepcopy()
 
@@ -286,9 +290,9 @@ def test_spectral_interpolate_reversed():
     np.testing.assert_almost_equal(sg.value, result.spectral_axis.value)
 
 
-def test_convolution_2D():
+def test_convolution_2D(data_55_delta):
 
-    proj, hdu = load_projection("55_delta.fits")
+    proj, hdu = load_projection(data_55_delta)
 
     # 1" convolved with 1.5" -> 1.8027....
     target_beam = Beam(1.802775637731995*u.arcsec, 1.802775637731995*u.arcsec,
@@ -307,27 +311,28 @@ def test_convolution_2D():
                                    conv_proj.value)
     assert conv_proj.beam == target_beam
 
+    # Pass a kwarg to the convolution function
+    conv_proj = proj.convolve_to(target_beam, nan_treatment='fill')
 
 
-def test_nocelestial_convolution_2D_fail():
+def test_nocelestial_convolution_2D_fail(data_255_delta, use_dask):
 
-    cube, data = cube_and_raw('255_delta.fits')
+    cube, data = cube_and_raw(data_255_delta, use_dask=use_dask)
 
     proj = cube.moment0(axis=1)
 
     test_beam = Beam(1.0 * u.arcsec)
 
-    with pytest.raises(WCSCelestialError) as exc:
+    with pytest.raises(WCSCelestialError,
+                       match="WCS does not contain two spatial axes."):
         proj.convolve_to(test_beam)
 
-    assert exc.value.args[0] == ("WCS does not contain two spatial axes.")
 
-
-def test_reproject_2D():
+def test_reproject_2D(data_55):
 
     pytest.importorskip('reproject')
 
-    proj, hdu = load_projection("55.fits")
+    proj, hdu = load_projection(data_55)
 
     wcs_in = WCS(proj.header)
     wcs_out = wcs_in.deepcopy()
@@ -352,23 +357,26 @@ def test_reproject_2D():
     assert result_wcs_from_header.wcs.compare(wcs_out.wcs)
 
 
-def test_nocelestial_reproject_2D_fail():
+def test_nocelestial_reproject_2D_fail(data_255_delta, use_dask):
 
     pytest.importorskip('reproject')
 
-    cube, data = cube_and_raw('255_delta.fits')
+    cube, data = cube_and_raw(data_255_delta, use_dask=use_dask)
 
     proj = cube.moment0(axis=1)
 
-    with pytest.raises(WCSCelestialError) as exc:
+    with pytest.raises(WCSCelestialError,
+                       match="WCS does not contain two spatial axes."):
         proj.reproject(cube.header)
 
-    assert exc.value.args[0] == ("WCS does not contain two spatial axes.")
-
 
 @pytest.mark.parametrize('use_memmap', (True,False))
-def test_downsample(use_memmap):
-    cube, data = cube_and_raw('255.fits')
+def test_downsample(use_memmap, data_255):
+
+    # FIXME: this test should be updated to use the use_dask fixture once
+    # DaskSpectralCube.downsample_axis is fixed.
+
+    cube, data = cube_and_raw(data_255, use_dask=False)
 
     dscube = cube.downsample_axis(factor=2, axis=0, use_memmap=use_memmap)
 
@@ -383,6 +391,7 @@ def test_downsample(use_memmap):
                          data[:,2:4,:].mean(axis=1),
                          data[:,4:,:].mean(axis=1), # just data[:,4,:]
                         ]).swapaxes(0,1)
+
     assert expected.shape == (2,3,5)
     assert dscube.shape == (2,3,5)
 
@@ -400,10 +409,13 @@ def test_downsample(use_memmap):
                                    dscube.filled_data[:].value)
 
 
-
 @pytest.mark.parametrize('use_memmap', (True,False))
-def test_downsample_wcs(use_memmap):
-    cube, data = cube_and_raw('255.fits')
+def test_downsample_wcs(use_memmap, data_255):
+
+    # FIXME: this test should be updated to use the use_dask fixture once
+    # DaskSpectralCube.downsample_axis is fixed.
+
+    cube, data = cube_and_raw(data_255, use_dask=False)
 
     dscube = (cube
               .downsample_axis(factor=2, axis=1, use_memmap=use_memmap)
@@ -423,6 +435,7 @@ def test_downsample_wcs(use_memmap):
 
     np.testing.assert_almost_equal(xpixnew_ypixnew, (0.75, 0.75))
 
+
 @pytest.mark.skipif('not tracemallocOK or (sys.version_info.major==3 and sys.version_info.minor<6) or not NPY_VERSION_CHECK')
 def test_reproject_3D_memory():
 
