@@ -80,6 +80,7 @@ import os
 on_travis = bool(os.environ.get('ON_TRAVIS'))
 
 from radio_beam import Beam, Beams
+from radio_beam.utils import BeamError
 
 NUMPY_LT_19 = LooseVersion(np.__version__) < LooseVersion('1.9.0')
 
@@ -718,6 +719,45 @@ class TestNumpyMethods(BaseTest):
                                 getattr(c2, method)(axis=axis, progressbar=True))
         self.c = self.d = None
 
+    @pytest.mark.parametrize('method', ('argmax_world', 'argmin_world'))
+    def test_transpose_arg_world(self, method, data_adv, data_vad, use_dask):
+        c1, d1 = cube_and_raw(data_adv, use_dask=use_dask)
+        c2, d2 = cube_and_raw(data_vad, use_dask=use_dask)
+
+        # The spectral axis should work in all of these test cases.
+        axis = 0
+        assert_allclose(getattr(c1, method)(axis=axis),
+                        getattr(c2, method)(axis=axis))
+        if not use_dask:
+            # check that all these accept progressbar kwargs
+            assert_allclose(getattr(c1, method)(axis=axis, progressbar=True),
+                            getattr(c2, method)(axis=axis, progressbar=True))
+
+        # But the spatial axes should fail since the pixel axes are correlated to
+        # the WCS celestial axes. Currently this will happen for ALL celestial axes.
+        for axis in [1, 2]:
+
+            with pytest.raises(utils.WCSCelestialError,
+                               match=re.escape(f"{method} requires the celestial axes")):
+
+                assert_allclose(getattr(c1, method)(axis=axis),
+                                getattr(c2, method)(axis=axis))
+
+        self.c = self.d = None
+
+    @pytest.mark.parametrize('method', ('argmax_world', 'argmin_world'))
+    def test_arg_world(self, method, data_adv, use_dask):
+        c1, d1 = cube_and_raw(data_adv, use_dask=use_dask)
+
+        # Pixel operation is same name with "_world" removed.
+        arg0_pixel = getattr(c1, method.split("_")[0])(axis=0)
+
+        arg0_world = np.take_along_axis(c1.spectral_axis[:, np.newaxis, np.newaxis],
+                                        arg0_pixel[np.newaxis, :, :], axis=0).squeeze()
+
+        assert_allclose(getattr(c1, method)(axis=0), arg0_world)
+
+        self.c = self.d = None
 
 class TestSlab(BaseTest):
 
@@ -1570,7 +1610,7 @@ def test_multibeam_custom(data_vda_beams, use_dask):
     new_beams = Beams([1.] * cube.shape[0] * u.deg)
 
     # Attach the beam
-    newcube = cube.with_beams(new_beams)
+    newcube = cube.with_beams(new_beams, raise_error_jybm=False)
 
     try:
         assert all(new_beams == newcube.beams)
@@ -1590,7 +1630,20 @@ def test_multibeam_custom_wrongshape(data_vda_beams, use_dask):
     new_beams = Beams([1.] * cube.shape[0] * u.deg)
 
     # Attach the beam
-    cube.with_beams(new_beams[:1])
+    cube.with_beams(new_beams[:1], raise_error_jybm=False)
+
+
+@pytest.mark.openfiles_ignore
+@pytest.mark.xfail(raises=utils.BeamUnitsError, strict=True)
+def test_multibeam_jybm_error(data_vda_beams, use_dask):
+
+    cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
+
+    # Make a new set of beams that differs from the original.
+    new_beams = Beams([1.] * cube.shape[0] * u.deg)
+
+    # Attach the beam
+    newcube = cube.with_beams(new_beams, raise_error_jybm=True)
 
 
 def test_multibeam_slice(data_vda_beams, use_dask):
@@ -1652,6 +1705,122 @@ def test_basic_unit_conversion_beams(data_vda_beams, use_dask):
                                     1e3))
 
 
+bunits_list = [u.Jy / u.beam, u.K, u.Jy / u.sr, u.Jy / u.pix, u.Jy / u.arcsec**2,
+               u.mJy / u.beam, u.mK]
+
+@pytest.mark.parametrize(('init_unit'), bunits_list)
+def test_unit_conversions_general(data_advs, use_dask, init_unit):
+
+    cube, data = cube_and_raw(data_advs, use_dask=use_dask)
+    cube._meta['BUNIT'] = init_unit.to_string()
+    cube._unit = init_unit
+
+    # Check all unit conversion combos:
+    for targ_unit in bunits_list:
+        newcube = cube.to(targ_unit)
+
+        if init_unit == targ_unit:
+            np.testing.assert_almost_equal(newcube.filled_data[:].value,
+                                           cube.filled_data[:].value)
+
+        else:
+            roundtrip_cube = newcube.to(init_unit)
+            np.testing.assert_almost_equal(roundtrip_cube.filled_data[:].value,
+                                           cube.filled_data[:].value)
+
+@pytest.mark.parametrize(('init_unit'), bunits_list)
+def test_multibeam_unit_conversions_general(data_vda_beams, use_dask, init_unit):
+
+    cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
+    cube._meta['BUNIT'] = init_unit.to_string()
+    cube._unit = init_unit
+
+    # Check all unit conversion combos:
+    for targ_unit in bunits_list:
+        newcube = cube.to(targ_unit)
+
+        if init_unit == targ_unit:
+            np.testing.assert_almost_equal(newcube.filled_data[:].value,
+                                           cube.filled_data[:].value)
+
+        else:
+            roundtrip_cube = newcube.to(init_unit)
+            np.testing.assert_almost_equal(roundtrip_cube.filled_data[:].value,
+                                           cube.filled_data[:].value)
+
+
+def test_beam_jpix_checks_array(data_advs, use_dask):
+    '''
+    Ensure round-trip consistency in our defined K -> Jy/pix conversions.
+
+    '''
+
+    cube, data = cube_and_raw(data_advs, use_dask=use_dask)
+    cube._meta['BUNIT'] = 'Jy / beam'
+    cube._unit = u.Jy/u.beam
+
+    jtok = cube.beam.jtok(cube.with_spectral_unit(u.GHz).spectral_axis)
+
+    pixperbeam = cube.pixels_per_beam * u.pix
+
+    cube_jypix = cube.to(u.Jy / u.pix)
+    np.testing.assert_almost_equal(cube_jypix.filled_data[:].value,
+                                   (cube.filled_data[:].value /
+                                    pixperbeam).value)
+
+    Kcube = cube.to(u.K)
+    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+                                   (cube_jypix.filled_data[:].value *
+                                    jtok[:,None,None] * pixperbeam).value)
+
+    # Round trips.
+    roundtrip_cube = cube_jypix.to(u.Jy / u.beam)
+    np.testing.assert_almost_equal(cube.filled_data[:].value,
+                                   roundtrip_cube.filled_data[:].value)
+
+    Kcube_from_jypix = cube_jypix.to(u.K)
+
+    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+                                   Kcube_from_jypix.filled_data[:].value)
+
+
+def test_multibeam_jpix_checks_array(data_vda_beams, use_dask):
+    '''
+    Ensure round-trip consistency in our defined K -> Jy/pix conversions.
+
+    '''
+
+    cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
+    cube._meta['BUNIT'] = 'Jy / beam'
+    cube._unit = u.Jy/u.beam
+
+    # NOTE: We are no longer using jtok_factors for conversions. This may need to be removed
+    # in the future
+    jtok = cube.jtok_factors()
+
+    pixperbeam = cube.pixels_per_beam * u.pix
+
+    cube_jypix = cube.to(u.Jy / u.pix)
+    np.testing.assert_almost_equal(cube_jypix.filled_data[:].value,
+                                   (cube.filled_data[:].value /
+                                    pixperbeam[:, None, None]).value)
+
+    Kcube = cube.to(u.K)
+    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+                                   (cube_jypix.filled_data[:].value *
+                                    jtok[:,None,None] *
+                                    pixperbeam[:, None, None]).value)
+
+    # Round trips.
+    roundtrip_cube = cube_jypix.to(u.Jy / u.beam)
+    np.testing.assert_almost_equal(cube.filled_data[:].value,
+                                   roundtrip_cube.filled_data[:].value)
+
+    Kcube_from_jypix = cube_jypix.to(u.K)
+
+    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+                                   Kcube_from_jypix.filled_data[:].value)
+
 
 def test_beam_jtok_array(data_advs, use_dask):
 
@@ -1659,21 +1828,14 @@ def test_beam_jtok_array(data_advs, use_dask):
     cube._meta['BUNIT'] = 'Jy / beam'
     cube._unit = u.Jy/u.beam
 
-    equiv = cube.beam.jtok_equiv(cube.with_spectral_unit(u.GHz).spectral_axis)
     jtok = cube.beam.jtok(cube.with_spectral_unit(u.GHz).spectral_axis)
 
-    Kcube = cube.to(u.K, equivalencies=equiv)
-    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
-                                   (cube.filled_data[:].value *
-                                    jtok[:,None,None]).value)
-
     # test that the beam equivalencies are correctly automatically defined
     Kcube = cube.to(u.K)
     np.testing.assert_almost_equal(Kcube.filled_data[:].value,
                                    (cube.filled_data[:].value *
                                     jtok[:,None,None]).value)
 
-
 def test_multibeam_jtok_array(data_vda_beams, use_dask):
 
     cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
@@ -1880,13 +2042,22 @@ def test_varyres_moment_logic_issue364(data_vda_beams, use_dask):
     assert_quantity_allclose(m0.meta['beam'].major, 0.35*u.arcsec)
 
 
-def test_mask_bad_beams(data_vda_beams, use_dask):
+@pytest.mark.skipif('not casaOK')
+@pytest.mark.parametrize('filename', ['data_vda_beams',
+                                      'data_vda_beams_image'],
+                         indirect=['filename'])
+def test_mask_bad_beams(filename, use_dask):
     """
     Prior to #543, this tested two different scenarios of beam masking.  After
     that, the tests got mucked up because we can no longer have minor>major in
     the beams.
     """
-    cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
+    if 'image' in str(filename) and not use_dask:
+        pytest.skip()
+
+    cube, data = cube_and_raw(filename, use_dask=use_dask)
+
+    assert isinstance(cube, base_class.MultiBeamMixinClass)
 
     # make sure all of the beams are initially good (finite)
     assert np.all(cube.goodbeams_mask)
@@ -1945,13 +2116,39 @@ def test_convolve_to(data_vda_beams, use_dask):
                                  nan_treatment='fill')
 
 
+def test_convolve_to_jybeam_onebeam(point_source_5_one_beam, use_dask):
+    cube, data = cube_and_raw(point_source_5_one_beam, use_dask=use_dask)
+
+    convolved = cube.convolve_to(Beam(10*u.arcsec))
+
+    # The peak of the point source should remain constant in Jy/beam
+    np.testing.assert_allclose(convolved[:, 5, 5].value, cube[:, 5, 5].value, atol=1e-5, rtol=1e-5)
+
+    assert cube.unit == u.Jy / u.beam
+
+
+def test_convolve_to_jybeam_multibeams(point_source_5_spectral_beams, use_dask):
+    cube, data = cube_and_raw(point_source_5_spectral_beams, use_dask=use_dask)
+
+
+    convolved = cube.convolve_to(Beam(10*u.arcsec))
+
+    # The peak of the point source should remain constant in Jy/beam
+    np.testing.assert_allclose(convolved[:, 5, 5].value, cube[:, 5, 5].value, atol=1e-5, rtol=1e-5)
+
+    assert cube.unit == u.Jy / u.beam
+
+
 def test_convolve_to_with_bad_beams(data_vda_beams, use_dask):
     cube, data = cube_and_raw(data_vda_beams, use_dask=use_dask)
 
     convolved = cube.convolve_to(Beam(0.5*u.arcsec))
 
-
-    with pytest.raises(ValueError,
+    # From: https://github.com/radio-astro-tools/radio-beam/pull/87
+    # updated exception to BeamError when the beam cannot be deconvolved.
+    # BeamError is not new in the radio_beam package, only its use here.
+    # Keeping the ValueError for testing against <v0.3.3 versions
+    with pytest.raises((BeamError, ValueError),
                        match="Beam could not be deconvolved"):
         # should not work: biggest beam is 0.4"
         convolved = cube.convolve_to(Beam(0.35*u.arcsec))
@@ -2162,6 +2359,32 @@ def test_spatial_smooth_t2d(data_adv, use_dask):
     np.testing.assert_almost_equal(cube_t2d[2].value, result2)
 
 
+@pytest.mark.openfiles_ignore
+@pytest.mark.parametrize('filename', ['point_source_5_one_beam', 'point_source_5_spectral_beams'],
+                         indirect=['filename'])
+@pytest.mark.xfail(raises=utils.BeamUnitsError, strict=True)
+def test_spatial_smooth_jybm_error(filename, use_dask):
+    '''Raise an error when Jy/beam units are getting spatially smoothed. This tests SCs and VRSCs'''
+
+    cube, data = cube_and_raw(filename, use_dask=use_dask)
+
+    # Tophat 2D smoothing test
+    t2d = Tophat2DKernel(3)
+    cube_t2d = cube.spatial_smooth(t2d)
+
+
+@pytest.mark.openfiles_ignore
+@pytest.mark.parametrize('filename', ['point_source_5_one_beam', 'point_source_5_spectral_beams'],
+                         indirect=['filename'])
+@pytest.mark.xfail(raises=utils.BeamUnitsError, strict=True)
+def test_spatial_smooth_median_jybm_error(filename, use_dask):
+    '''Raise an error when Jy/beam units are getting spatially median smoothed. This tests SCs and VRSCs'''
+
+    cube, data = cube_and_raw(filename, use_dask=use_dask)
+
+    cube_median = cube.spatial_smooth_median(3)
+
+
 def test_spatial_smooth_median(data_adv, use_dask):
 
     pytest.importorskip('scipy.ndimage')
@@ -2378,7 +2601,7 @@ def test_mask_none(use_dask):
 
 
 @pytest.mark.parametrize('filename', ['data_vda', 'data_vda_beams'],
-                            indirect=['filename'])
+                         indirect=['filename'])
 def test_mask_channels_preserve_mask(filename, use_dask):
 
     # Regression test for a bug that caused the mask to not be preserved.
@@ -2422,3 +2645,55 @@ def test_minimal_subcube(use_dask):
     subcube = cube.minimal_subcube()
 
     assert subcube.shape == (3, 5, 2)
+
+
+def test_minimal_subcube_nomask(use_dask):
+
+    if not use_dask:
+        pytest.importorskip('scipy')
+
+    data = np.arange(210, dtype=float).reshape((5, 6, 7))
+
+    wcs = WCS(naxis=3)
+    wcs.wcs.ctype = ['RA---TAN', 'DEC--TAN', 'VELO-HEL']
+
+    cube = SpectralCube(data * u.Jy / u.beam, wcs=wcs, use_dask=use_dask)
+
+    # verify that there is no mask
+    assert cube._mask is None
+
+    # this should not raise an Exception
+    subcube = cube.minimal_subcube()
+
+    # shape is unchanged
+    assert subcube.shape == (5, 6, 7)
+
+
+def test_regression_719(data_adv, use_dask):
+    """
+    Issue 719: exception raised when checking for beam
+    """
+    cube, data = cube_and_raw(data_adv, use_dask=use_dask)
+
+    # force unit for use below
+    cube._unit = u.Jy/u.beam
+
+    assert hasattr(cube, 'beam')
+
+    slc = cube[0,:,:]
+
+    # check that the hasattr tests work
+    from .. cube_utils import _has_beam, _has_beams
+
+    assert _has_beam(slc)
+    assert not _has_beams(slc)
+
+    # regression test: full example that broke
+    mx = cube.max(axis=0)
+    beam = cube.beam
+    cfrq = 100*u.GHz
+
+    # This should not raise an exception
+    mx_K = (mx*u.beam).to(u.K,
+                          u.brightness_temperature(beam_area=beam,
+                                                   frequency=cfrq))
